# -*- coding: utf-8 -*-
#
# (c) 2008 by Intevation GmbH
# This is Free software under the GPLv3. 
# See LICENSE comming with the source of 'mpuls offline'
# for details.
#
# author: Sascha L. Teichmann <sascha.teichmann@intevation.de>
#

import PyQt4.QtCore   as QtCore
import PyQt4.QtGui    as QtGui
import PyQt4.QtWebKit as QtWebKit

import os
import traceback

from cgi import parse_qs

from string import Template

from lib.renderer     import FormDataImpl, ViewRenderer, ErrorRenderer
from lib.htmltemplate import HTML_TEMPLATE, EMPTY_HTML, HELP_TEMPLATE
from lib.filecache    import UTF8

from model.data   import PageNode
from model.memory import MemoryDatastore

from ui.dialogs   import DefaultsDialog, AboutDialog, show_help

TITLE = "mpuls WASKO Offline-Klient"

class MainWindow(QtGui.QMainWindow):

    def __init__(self, main):

        QtGui.QMainWindow.__init__(self)

        self.setObjectName("python")

        self.resize(*self.initialSize())

        self.main = main
        self.store_directory = None

        self.setup_menu_items()
        self.web = QtWebKit.QWebView(self)

        icon_path = os.path.join(
            main.file_cache.get_base(), 'images', 'icons', 'app-icon.png')

        self.setWindowIcon(QtGui.QIcon(icon_path))

        self.dump_html = os.getenv('MPULS_OFFLINE_DUMP_HTML') and \
            self.__dump_html or self.__dump_null

        self.wireWithHTML()
        self.setInitialPage()

        self.setCentralWidget(self.web)
        self.center()
        self.update_window_title()

    def setup_menu_items(self):

        new_case = QtGui.QAction('Neu', self)
        new_case.setShortcut('Ctrl+N')
        new_case.setStatusTip('Neue Akte anlegen')
        self.connect(
            new_case,
            QtCore.SIGNAL('triggered()'),
            QtCore.SLOT('new_case()'))

        load_case = QtGui.QAction('Laden...', self)
        load_case.setShortcut('Ctrl+O')
        load_case.setStatusTip('Akte laden')
        self.connect(
            load_case,
            QtCore.SIGNAL('triggered()'),
            QtCore.SLOT('load_case()'))

        save_case = QtGui.QAction('Speichern', self)
        save_case.setShortcut('Ctrl+S')
        save_case.setStatusTip('Akte speichern')
        self.connect(
            save_case,
            QtCore.SIGNAL('triggered()'),
            QtCore.SLOT('save_case()'))

        save_as_case = QtGui.QAction('Speichern unter...', self)
        save_as_case.setShortcut('Ctrl+Shift+S')
        save_as_case.setStatusTip('Akte unter anderem Namen speichern')
        self.connect(
            save_as_case, 
            QtCore.SIGNAL('triggered()'),
            QtCore.SLOT('save_as_case()'))

        quit = QtGui.QAction('Beenden', self)
        quit.setShortcut('Ctrl+Q')
        quit.setStatusTip('Programm beenden')
        self.connect(
            quit,
            QtCore.SIGNAL('triggered()'),
            QtCore.SLOT('close()'))

        if os.getenv('MPULS_OFFLINE_PRINT_PAGE'):
            printing = QtGui.QAction('Drucken', self)
            printing.setShortcut('Ctrl+P')
            printing.setStatusTip('Seite drucken...')
            self.connect(
                printing,
                QtCore.SIGNAL('triggered()'),
                QtCore.SLOT('print_page()'))
        else:
            printing = None

        menubar = self.menuBar()
        file = menubar.addMenu('&Datei')
        file.addAction(new_case)
        file.addAction(load_case)
        file.addAction(save_case)
        file.addAction(save_as_case)

        if printing:
            file.addSeparator()
            file.addAction(printing)

        file.addSeparator()

        file.addAction(quit)

        defaults = QtGui.QAction('Feldvorgaben...', self)
        defaults.setShortcut('Ctrl+E')
        defaults.setStatusTip(u'Vorgaben für Felder in der Akte')
        self.connect(
            defaults, 
            QtCore.SIGNAL('triggered()'),
            QtCore.SLOT('show_defaults_dialog()'))

        edit = menubar.addMenu('&Einstellungen')
        edit.addAction(defaults)

        about = QtGui.QAction(u'Über...', self)
        about.setStatusTip(u'Über den Offline-Klienten')
        self.connect(
            about, 
            QtCore.SIGNAL('triggered()'),
            QtCore.SLOT('about()'))

        help = menubar.addMenu('&Hilfe')
        help.addAction(about)

    def wireWithHTML(self):
        frame = self.web.page().mainFrame()
        self.connect(
            frame,
            QtCore.SIGNAL('javaScriptWindowObjectCleared()'),
            QtCore.SLOT('register_javascript()'))

    @QtCore.pyqtSignature("void")
    def register_javascript(self):
        frame = self.web.page().mainFrame()
        frame.addToJavaScriptWindowObject("python", self)

    def isHTMLFormModified(self):
        '''Returns true if the values in the formuar of the HTML page have changed'''
        return self.call_javascript('isModified()').toBool()

    def initialSize(self):
        screen = QtGui.QDesktopWidget().screenGeometry()
        size   = self.geometry()
        return (
            min(1024, screen.width()), 
            min(718, max(10, screen.height()-50)))

    def center(self):
        screen = QtGui.QDesktopWidget().screenGeometry()
        size   = self.geometry()
        self.move(
            (screen.width()  - size.width())/2, 
            (screen.height() - size.height())/2)

    def update_window_title(self):

        document = self.main.document

        if not document:
            title = TITLE
        else:
            filename = document.getFilename()
            filename = not filename and "Kein Dateiname" or \
                os.path.basename(filename)
            dirty = document.isDirty() and " [modifiziert]" or ""
            title = u"%s - %s%s" % (TITLE, filename, dirty)

        self.setWindowTitle(title)

    def unsaved_data(self):
        return QtGui.QMessageBox.question(
            self,
            "Programm wirklich beenden?",
            u"Sie haben noch ungesicherte Daten.\n"
                u"Wollen Sie das Programm wirklich beenden?\n"
                u"Dabei könnten wichtige Änderungen verloren gehen.",
            QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)

    def unsaved_new_data(self):
        return QtGui.QMessageBox.question(
            self,
            "Neue Akte wirklich anlegen?",
            u"Sie haben noch ungesicherte Daten.\n"
                u"Wollen Sie wirklich eine neue Akte anlegen?\n"
                u"Dabei könnten wichtige Änderungen verloren gehen.",
            QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)

    def unsaved_load_data(self):
        return QtGui.QMessageBox.question(
            self,
            "Akte wirklich laden?",
            u"Sie haben noch ungesicherte Daten.\n"
                u"Wollen Sie wirklich eine andere Akte laden?\n"
                u"Dabei könnten wichtige Änderungen verloren gehen.",
            QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)

    @QtCore.pyqtSignature("QCloseEvent")
    def closeEvent(self, evt):
        document = self.main.document
        if document and document.isDirty() or self.isHTMLFormModified():
            reply = self.unsaved_data()
            if reply == QtGui.QMessageBox.Yes:
                evt.accept()
            else:
                evt.ignore()
                return
        # cleanup swap-file
        swapfile = os.path.join(self.main.config.get_homedir(), self.main.config.get_swapfile())
        try:
            os.remove(swapfile)
        except OSError, e:
            pass

        QtGui.QMainWindow.closeEvent(self, evt)

    @QtCore.pyqtSignature('void')
    def new_case(self):
        document = self.main.document
        if document and document.isDirty() or self.isHTMLFormModified():
            reply = self.unsaved_new_data()
            if reply == QtGui.QMessageBox.No:
                return
        swapfile = os.path.join(self.main.config.get_homedir(), self.main.config.get_swapfile())
        self.main.document = MemoryDatastore(self.main.formed, self.main.defaults)
        self.main.document.setSwapfile(swapfile)
        self.setInitialPage()
        self.update_window_title()

    @QtCore.pyqtSignature('void')
    def load_case(self):
        document = self.main.document
        if document and document.isDirty() or self.isHTMLFormModified():
            reply = self.unsaved_load_data()
            if reply == QtGui.QMessageBox.No:
                return

        filename = QtGui.QFileDialog.getOpenFileName(
            self,
            "Bitte Dateinamen angeben.",
            self.get_store_directory(),
            "Fallakten (*.xml)")

        if not filename:
            return

        swapfile = os.path.join(self.main.config.get_homedir(),self.main.config.get_swapfile())
        filename = unicode(filename)

        self.set_store_directory(filename)

        try:
            document = MemoryDatastore(self.main.formed, self.main.defaults, filename)
        except:
             QtGui.QMessageBox.critical(
                self,
                "Fehler beim Laden",
                u"Beim Laden der Fallakte ist ein Fehler aufgetreten:\n%s" % \
                    traceback.format_exc())
             return

        document.setSwapfile(swapfile)
        self.main.document = document
        self.setInitialPage()
        self.update_window_title()

    def get_store_directory(self):
        if not self.store_directory:
            self.store_directory = os.path.expanduser('~')
        return self.store_directory

    def set_store_directory(self, file):
        self.store_directory = os.path.dirname(file)

    @QtCore.pyqtSignature('void')
    def save_case(self):
        document = self.main.document
        if not document and not document.isDirty():
            return

        filename = document.getFilename()

        if not filename:
            filename = QtGui.QFileDialog.getSaveFileName(
                self,
                "Bitte Dateinamen angeben.",
                self.get_store_directory(),
                "Fallakten (*.xml)")

            if not filename:
                return

            filename = unicode(filename)
            if not filename.lower().endswith(u'.xml'):
                filename += u'.xml'

            self.set_store_directory(filename)
            document.setFilename(filename)

        try:
            document.store()
        except:
             QtGui.QMessageBox.critical(
                self,
                "Fehler beim Absspeichern",
                u"Beim Abspeichern der Fallakte ist ein Fehler aufgetreten:\n%s" % \
                    traceback.format_exc())
        self.update_window_title()
        
    @QtCore.pyqtSignature('void')
    def print_page(self):
        document = self.main.document
        if not document or not self.web: return
        printer = QtGui.QPrinter()
        dialog = QtGui.QPrintDialog(printer, self)
        if dialog.exec_() == QtGui.QDialog.Accepted:
            self.web.print_(printer)

    @QtCore.pyqtSignature('void')
    def save_as_case(self):
        document = self.main.document
        if not document: return

        filename = document.getFilename()

        if not filename:
            filename = self.get_store_directory()

        filename = QtGui.QFileDialog.getSaveFileName(
            self,
            "Bitte Dateinamen angeben.",
            filename,
            "Fallakten (*.xml)")

        if not filename:
            return

        filename = unicode(filename)
        if not filename.lower().endswith(u'.xml'):
            filename += u'.xml'

        self.set_store_directory(filename)
        document.setFilename(filename)

        try:
            document.store()
        except:
             QtGui.QMessageBox.critical(
                self,
                "Fehler beim Absspeichern",
                u"Beim Abspeichern der Fallakte ist ein Fehler aufgetreten:\n%s" % \
                    traceback.format_exc())
        self.update_window_title()

    @QtCore.pyqtSignature('void')
    def about(self):
        about = AboutDialog(
            self.main.file_cache.get_file("about.html", UTF8),
            self)
        about.setModal(True)
        about.exec_()

    @QtCore.pyqtSignature('void')
    def show_defaults_dialog(self):
        dialog = DefaultsDialog(
            self.main.formed,
            self.main.defaults, 
            self)
        dialog.setModal(True)
        dialog.exec_()

    def render_page(self, page_name, unfold = False):

        document = self.main.document

        if not page_name or not document:
            html = EMPTY_HTML
        else:
            page = self.main.formed.findByClassAndName(PageNode, page_name)
            if not page:
                html = EMPTY_HTML
            else:
                file_cache = self.main.file_cache
                errors          = document.getErrors()
                warnings        = document.warningsOnPage(page_name)
                error_renderer  = ErrorRenderer(errors, warnings, file_cache)
                formular_errors = error_renderer.render(page)

                form_data = FormDataImpl(
                    file_cache,
                    self.main.annotations,
                    document, 
                    errors,
                    warnings,
                    [page])

                navigation = document.getNavigation()

                form_renderer = ViewRenderer(form_data, ro_mode = False)
                waska_form    = form_renderer.renderView(
                    page_name, 
                    navigation.getNextPage(page_name) is not None)

                ti = navigation.getTreeItem(page_name)

                if ti:
                    form_header = ti.renderHeader(form_data)
                    if unfold: ti.unfold()
                else:
                    form_header = ""

                navigation_html = navigation.render(
                    page_name,
                    file_cache,
                    document.getPageNamesWithErrors())

                html = HTML_TEMPLATE.safe_substitute(
                    JAVA_SCRIPT_PYTHON = file_cache.get_file("js/python.js",   UTF8),
                    JAVA_SCRIPT_JQUERY = file_cache.get_file("js/jquery.js",   UTF8),
                    ALL_CSS            = file_cache.get_file("styles/all.css", UTF8),
                    COLOR_CSS          = file_cache.get_file("styles/color3.css", UTF8),
                    SCREEN_CSS         = file_cache.get_file("styles/screen.css", UTF8),
                    NAVIGATION         = navigation_html,
                    FORM_HEADER        = form_header,
                    FORMULAR_ERRORS    = formular_errors,
                    WASKA_FORM         = waska_form)

        # Uncomment this if you want to dump the HTML to files
        self.dump_html(html)
        self.web.setHtml(html, QtCore.QUrl(self.main.base_url))

    def __dump_null(self, html):
        pass

    def __dump_html(self, html):
        if not hasattr(self, 'dump_html_counter'):
            self.dump_html_counter = 0
        import codecs
        f = None
        try:
            f = codecs.open(
                "html-dump-%03d.html" % self.dump_html_counter, 
                "w", "UTF-8")
            f.write(html)
        finally:
            if f: 
                try: f.close()
                except: pass

        self.dump_html_counter += 1

    def setInitialPage(self):
        name = None
        for page in self.main.formed.findAllByClass(PageNode):
            name = page.getName()
            break
        self.render_page(name)

    def call_javascript(self, cmd):
        return self.web.page().mainFrame().evaluateJavaScript(cmd)

    def save_render(self, values, goto_next):
        document = self.main.document
        if not document or not values: return

        values = str(values) # treat as bytes!

        params = parse_qs(values, keep_blank_values=1)

        page = params['page'][0]

        document.save(params, page)

        if goto_next:
            next = document.getNavigation().getNextPage(page)
            if next: page = next

        self.render_page(page, goto_next)
        self.update_window_title()

    @QtCore.pyqtSignature('QString')
    def save(self, page):
        self.save_render(page, False)
        self.call_javascript('changed = 0')

    @QtCore.pyqtSignature('QString')
    def save_next(self, page):
        self.save_render(page, True)

    @QtCore.pyqtSignature('QString')
    def select_item(self, page):
        self.render_page(unicode(page))

    @QtCore.pyqtSignature('QString, QString')
    def close_branch(self, page, old_page):
        document = self.main.document
        if not document: return
        navigation = document.getNavigation()
        ti = navigation.getTreeItem(page)
        if not ti: return
        ti.set_open(False)
        self.render_page(unicode(old_page))

    @QtCore.pyqtSignature('QString, QString')
    def open_branch(self, page, old_page):
        document = self.main.document
        if not document: return
        navigation = document.getNavigation()
        ti = navigation.getTreeItem(page)
        if not ti: return
        ti.set_open(True)
        self.render_page(unicode(old_page))

    @QtCore.pyqtSignature('QString')
    def select_branch(self, page):
        self.render_page(unicode(page), True)

    @QtCore.pyqtSignature('QString, QString')
    def undo_error(self, item_id, page):
        document = self.main.document
        if not document: return
        document.getErrors().pop(unicode(item_id), None)
        self.render_page(unicode(page))

    def first_description(self, item):
        n = self.main.formed.findByName(item)
        if not n: return ""
        while n:
            description = n.getDescription()
            if description: return description
            n = n.parent

    @QtCore.pyqtSignature('QString')
    def required_help(self, item):
        item = unicode(item)
        content = self.main.annotations.getAnnotations(item)
        if content: 
            show_help(HELP_TEMPLATE.safe_substitute(
                KIND    = u"Erforderliches Feld",
                TITLE   = self.first_description(item),
                CONTENT = content), self)

    @QtCore.pyqtSignature('QString')
    def eval_help(self, item):
        item = unicode(item)
        content = self.main.annotations.getEvaluation(item)
        if content: 
            show_help(HELP_TEMPLATE.safe_substitute(
                KIND    = u"Auswertungsrelevantes Feld",
                TITLE   = self.first_description(item),
                CONTENT = content), self)

    @QtCore.pyqtSignature('QString')
    def help(self, item):
        item = unicode(item)
        content = self.main.annotations.getHelp(item)
        if content: 
            show_help(HELP_TEMPLATE.safe_substitute(
                KIND    = u"Hilfe",
                TITLE   = self.first_description(item),
                CONTENT = content), self)

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
