# -*- coding: latin1 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
# 
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS - 
# Web-Anwendungs-Server fuer Kompetenzagenturen).
# 
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
# 
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
# 
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
# 
# mpuls WASKA has been developed on behalf of the 
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and 
# European Social Fund resources.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
#

from xml.sax          import make_parser
from xml.sax.saxutils import quoteattr
from parser           import SAXBuilder

import codecs

from factories import DocumentFactory, SimpleCreator, NodeComponentFactory, NodeFactory
from offline.model.nodecomponents import RootNode, Document, Leaf, Node
from offline.model.data           import *

DOCUMENT_TAG        = u"document"
PAGE_TAG            = u"page"
GROUP_TAG           = u"group"
MATRIX_TAG          = u"radio-matrix"
RADIO_TAG           = u"radio"
BOOL_TAG            = u"bool"
INT_TAG             = u"int"
INFO_TAG            = u"info"
TEXT_TAG            = u"text"
CHOICE_TAG          = u"choice"
EXT_CHOICE_LIST_TAG = u"external-choice"
DATE_TAG            = u"date"
TEXT_AREA_TAG       = u"textarea"
RULE_TAG            = u"rule"

namesToClasses = {
    DOCUMENT_TAG:        RootNode,
    PAGE_TAG:            PageNode,
    GROUP_TAG:           GroupNode,
    RADIO_TAG:           RadioNode,
    MATRIX_TAG:          MatrixNode,
    BOOL_TAG:            BoolLeaf,
    CHOICE_TAG:          ChoiceNode,
    EXT_CHOICE_LIST_TAG: ExternalChoiceListLeaf,
    INT_TAG:             IntLeaf,
    INFO_TAG:            InfoLeaf,
    TEXT_TAG:            TextLeaf,
    TEXT_AREA_TAG:       TextAreaLeaf,
    RULE_TAG:            RuleLeaf,
    DATE_TAG:            DateLeaf}

factoryCreators = {
    DOCUMENT_TAG:        SimpleCreator(DocumentFactory,      RootNode),
    PAGE_TAG:            SimpleCreator(NodeFactory,          PageNode),
    GROUP_TAG:           SimpleCreator(NodeFactory,          GroupNode),
    MATRIX_TAG:          SimpleCreator(NodeFactory,          MatrixNode),
    RADIO_TAG:           SimpleCreator(NodeFactory,          RadioNode),
    CHOICE_TAG:          SimpleCreator(NodeFactory,          ChoiceNode),
    EXT_CHOICE_LIST_TAG: SimpleCreator(NodeComponentFactory, ExternalChoiceListLeaf),
    BOOL_TAG:            SimpleCreator(NodeComponentFactory, BoolLeaf),
    INT_TAG:             SimpleCreator(NodeComponentFactory, IntLeaf),
    INFO_TAG:            SimpleCreator(NodeComponentFactory, InfoLeaf),
    TEXT_AREA_TAG:       SimpleCreator(NodeComponentFactory, TextAreaLeaf),
    TEXT_TAG:            SimpleCreator(NodeComponentFactory, TextLeaf),
    RULE_TAG:            SimpleCreator(NodeComponentFactory, RuleLeaf),
    DATE_TAG:            SimpleCreator(NodeComponentFactory, DateLeaf)}

classesToTagNames = dict([(v, k) for k, v in namesToClasses.iteritems()])

def openDocument(path):
    builder = SAXBuilder(path)

    builder.setCreators(factoryCreators)
    f = None
    try:
        f = file(path, "rb")
        parser = make_parser()
        parser.setContentHandler(builder)
        parser.parse(f)
        return builder.getDocument()
    finally:
        if f is not None:
            try:    f.close()
            except: pass

def _attributesToXML(nc):
    out = []
    for key, value in nc.attributes.iteritems():
        if value:
            out.append(u" %s=%s" % (key, quoteattr(value)))
    return " ".join(out)

def _toXML(nc):
    if isinstance(nc, Node):
        tagName = classesToTagNames.get(nc.__class__, "node")
        out = [ u"<%s%s>" % (tagName, _attributesToXML(nc)) ]
        for child in nc.children:
            out.append(_toXML(child))
        out.append(u"</%s>" % tagName)
        return "".join(out)
    if isinstance(nc, Leaf):
        tagName = classesToTagNames.get(nc.__class__, "leaf")
        return u"<%s%s/>" % (tagName, _attributesToXML(nc))
    if isinstance(nc, Document):
        out = [ u'<?xml version="1.0" encoding="UTF-8"?>\n' ]
        out.append(_toXML(nc.root))
        return "".join(out)
    return ""

def saveDocument(document, path):
    f = None
    try:
        f = codecs.open(path, "wb", "UTF-8")
        print >> f, _toXML(document)
    finally:
        if f is not None:
            try: f.close()
            except: pass

# vim:set ts=4 sw=4 si et sta sts=4:
